import os
from cffi import FFI
import json

ffi = FFI()

ffi.cdef("""
char* NaturalSortedVersions(char* versions, _Bool descending, _Bool safe_parse);
char* AnalyzeConstrains(char* language, char* constraints, char* versions);
void FreeCString(char* str);
""", override=True)  # allows reloads without error


lib_path = os.path.join(os.path.dirname(__file__), "libpyversions.so")
C = ffi.dlopen(lib_path)

def natural_sorted_versions(versions: list, descending: bool = False, safe_parse: bool = True) -> list[str]:
    versions_json = json.dumps(versions).encode("utf-8")
    c_data = ffi.new("char[]", versions_json)

    res = C.NaturalSortedVersions(c_data, descending, safe_parse)
    py_res_str = ffi.string(res).decode()

    C.FreeCString(res)

    if py_res_str.startswith("ERROR:"):
        raise RuntimeError(py_res_str)

    return json.loads(py_res_str)

def analyze_constraints(language: str, constraints: str, versions: list[str]) -> list[str]:
    # Convert Python list to JSON string
    versions_json = json.dumps(versions).encode("utf-8")

    # Convert Python strings to C strings
    c_language = ffi.new("char[]", language.encode("utf-8"))
    c_constraints = ffi.new("char[]", constraints.encode("utf-8"))
    c_versions = ffi.new("char[]", versions_json)

    # Call the C function
    res = C.AnalyzeConstrains(c_language, c_constraints, c_versions)
#     res = C.AnalyzeConstrains(
#          ffi.cast("char*", c_language),
#          ffi.cast("char*", c_constraints),
#          ffi.cast("char*", c_versions),
#      )

    # Convert returned C string to Python string
    py_res_str = ffi.string(res).decode("utf-8")

    # Free C string memory
    C.FreeCString(res)

    # If Go returned an error string, you can optionally raise an exception
    if py_res_str.startswith("ERROR:"):
        raise RuntimeError(py_res_str)

    # Decode JSON string into Python list
    return json.loads(py_res_str)




from setuptools import setup, find_packages
import io
import os


def read_long_description():
    readme_path = os.path.join(os.path.dirname(__file__), "README.md")
    try:
        with io.open(readme_path, encoding="utf-8") as f:
            return f.read()
    except FileNotFoundError:
        return ""


setup(
    name="semverish",
    version="2.0.1",
    author="Al Arafat Tanin",
    author_email="arafat.rng70@gmail.com",
    description=(
        "Python wrapper around Go module `github.com/rng70/versions/v2` "
        "for parsing and sorting semver and non-semver-ish versions and constraints."
    ),
    long_description=read_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/rng70/versions",
    project_urls={
        "Bug Tracker": "https://github.com/rng70/versions/issues",
        "Source": "https://github.com/rng70/versions",
        "Changelog": "https://github.com/rng70/versions/releases",
    },
    license="MIT",
    license_expression="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Go",
        "Operating System :: POSIX :: Linux",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Version Control",
    ],
    keywords="semver version sorting constraints npm pypi nuget maven",
    packages=find_packages(exclude=("tests*", "docs*")),
    include_package_data=True,
    package_data={
        "semverish": ["libpyversions.so"],
    },
    install_requires=[
        "cffi>=1.15.0",
    ],
    python_requires=">=3.8",
    zip_safe=False,
)
